//
//  TiddlyWikiSaver.m
//  TiddlyWikiSaver
//
//  Created by Jonathan Paisley on 14/05/2005.
//  Copyright 2005. All rights reserved.
//
// $Id: TiddlyWikiSaver.m 770 2005-05-14 16:09:36Z jp $

#import "TiddlyWikiSaver.h"
#import <WebKit/WebKit.h>

static NSMutableDictionary *sAcknowledgedPaths;
static BOOL sDoAcknowledgePaths = YES;

#define kTWSAcknowledgeRead 1
#define kTWSAcknowledgeWrite 2

@implementation TiddlyWikiSaver

+ (NSView *)plugInViewWithArguments:(NSDictionary *)arguments
{
	if (!sAcknowledgedPaths) {
		sAcknowledgedPaths = [[NSMutableDictionary alloc] initWithCapacity: 0];
	}
    TiddlyWikiSaver *view = [[[self alloc] init] autorelease];
    [view setArguments:arguments];
    return view;
}

- (void)dealloc
{   
    [_arguments release];
    [super dealloc];
}

- (void)setArguments:(NSDictionary *)arguments
{
    [arguments copy];
    [_arguments release];
    _arguments = arguments;
}

- (void)webPlugInInitialize
{
}

- (void)webPlugInStart
{
}

- (void)webPlugInStop
{
}

- (void)webPlugInDestroy
{
}

- (void)webPlugInSetIsSelected:(BOOL)isSelected
{
}

// Scripting support

+ (BOOL)isSelectorExcludedFromWebScript:(SEL)selector
{
    if (selector == @selector(readURL:) || selector == @selector(writeString:toURL:)) {
        return NO;
    }
    return YES;
}

+ (NSString*)webScriptNameForSelector:(SEL)selector
{
	if (selector == @selector(readURL:)) {
		return @"readURL";
	} else if (selector == @selector(writeString:toURL:)) {
		return @"writeStringToURL";
	}
	return nil;
}

+ (BOOL)isKeyExcludedFromWebScript:(const char *)property
{
    //if (strcmp(property, "muted") == 0) {
    //    return NO;
    //}
    return YES;
}

- (id)objectForWebScript
{
    return self;
}

- (BOOL)canAccessURL:(NSURL*)url forWriting:(BOOL)writing
{
	NSURL *baseURL = [_arguments objectForKey:WebPlugInBaseURLKey];
	
	// For now, policy is that it must be the the current web page
	return [url isEqual: baseURL];
}

- (NSURL*)validateURL:(NSString*)urlString forWriting:(BOOL)writing
{
	if ([urlString length] != 0) {
		NSURL *baseURL = [_arguments objectForKey:WebPlugInBaseURLKey];
		NSURL *url = [[NSURL URLWithString:urlString relativeToURL:baseURL] standardizedURL];
		
		if ([[url scheme] isEqualToString: @"file"] && [self canAccessURL:url forWriting:writing]) {
			if (sDoAcknowledgePaths) {
				NSNumber *n = [sAcknowledgedPaths objectForKey: url];
				int mask = writing ? kTWSAcknowledgeWrite : kTWSAcknowledgeRead;
				int cur = [n intValue];
				if (!(cur & mask)) {
					NSAlert *alert = [[NSAlert alloc] init];
					[alert addButtonWithTitle:@"Deny"];
					[[alert window] setInitialFirstResponder: [alert addButtonWithTitle:@"Allow Once"]];
					[alert addButtonWithTitle:@"Allow For Session"];
					[alert setMessageText:@"TiddlyWiki File Saver: Confirm file access"];
					[alert setInformativeText:
						[NSString stringWithFormat: @"Do you want to allow %s access to the following URL?:\n\n%@",
							 writing ? "write" : "read",url]];
					[alert setAlertStyle:NSWarningAlertStyle];
					int result = [alert runModal];
					switch (result) {
						case NSAlertFirstButtonReturn:
							return nil;
						
						case NSAlertSecondButtonReturn:
							break;
						
						case NSAlertThirdButtonReturn:
							cur |= mask;
							[sAcknowledgedPaths setObject: [NSNumber numberWithInt: cur] forKey: url];
							break;
					}		
				}
				
			}
			return url;
		} else {
			NSAlert *alert = [[NSAlert alloc] init];
			[alert addButtonWithTitle:@"OK"];
			[alert setMessageText:@"TiddlyWiki File Saver Plugin"];
			[alert setInformativeText:
				[NSString stringWithFormat: @"An attempt was denied to %s the following URL:\n\n%@",
					writing ? "write to" : "read",url]];
			[alert setAlertStyle:NSWarningAlertStyle];
			[alert runModal];
		}
	}
	return nil;
}

- (id)readURL:(NSString*)urlString
{
	if (![urlString isKindOfClass: [NSString class]])
		return nil; //[NSNull null]; // doesn't seem to be getting converted to JavaScript null properly

	NSURL *url = [self validateURL: urlString forWriting: NO];
	if (url) {
		NSData *data = [NSData dataWithContentsOfURL: url]; 
		if (data) {
			return [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease];
		}
	}
	return nil; //[NSNull null];
}

- (BOOL)writeString:(NSString*)text toURL:(NSString*)urlString
{
	if (![text isKindOfClass: [NSString class]] || ![urlString isKindOfClass: [NSString class]])
		return NO;
	
	NSURL *url = [self validateURL: urlString forWriting: YES];
	if (url) {
		NSData *data = [text dataUsingEncoding:NSUTF8StringEncoding];
		return [data writeToURL:url atomically:YES];
	}
	return NO;
}

@end
